﻿//------------------------------------------------------------------------------
// <copyright file="MainWindow.xaml.cs" company="Microsoft">
//     Copyright (c) Microsoft Corporation.  All rights reserved.
// </copyright>
//------------------------------------------------------------------------------

namespace Microsoft.Samples.Kinect.SkeletonBasics
{
    using System;
    using System.IO;
    using System.IO.Ports;
    using System.Windows;
    using System.Windows.Media;
    using Microsoft.Kinect;    

    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        /// <summary>
        /// Width of output drawing
        /// </summary>
        private const float RenderWidth = 640.0f;

        /// <summary>
        /// Height of our output drawing
        /// </summary>
        private const float RenderHeight = 480.0f;

        /// <summary>
        /// Thickness of drawn joint lines
        /// </summary>
        private const double JointThickness = 3;

        /// <summary>
        /// Thickness of body center ellipse
        /// </summary>
        private const double BodyCenterThickness = 10;

        /// <summary>
        /// Thickness of clip edge rectangles
        /// </summary>
        private const double ClipBoundsThickness = 10;

        /// <summary>
        /// Brush used to draw skeleton center point
        /// </summary>
        private readonly Brush centerPointBrush = Brushes.Blue;

        /// <summary>
        /// Brush used for drawing joints that are currently tracked
        /// </summary>
        private readonly Brush trackedJointBrush = new SolidColorBrush(Color.FromArgb(255, 68, 192, 68));

        /// <summary>
        /// Brush used for drawing joints that are currently inferred
        /// </summary>        
        private readonly Brush inferredJointBrush = Brushes.Yellow;

        /// <summary>
        /// Pen used for drawing bones that are currently tracked
        /// </summary>
        private readonly Pen trackedBonePen = new Pen(Brushes.Green, 6);

        /// <summary>
        /// Pen used for drawing bones that are currently inferred
        /// </summary>        
        private readonly Pen inferredBonePen = new Pen(Brushes.Gray, 1);

        /// <summary>
        /// Active Kinect sensor
        /// </summary>
        private KinectSensor sensor;

        /// <summary>
        /// Drawing group for skeleton rendering output
        /// </summary>
        private DrawingGroup drawingGroup;

        /// <summary>
        /// Drawing image that we will display
        /// </summary>
        private DrawingImage imageSource;

        static SerialPort _serialPort;

        static string angA = "0", angB = "0", angAL = "0", angBL = "0";
        static Boolean trackStart = false;

        long Time_i = 0;
        long Time_now = 0;

        /// <summary>
        /// Initializes a new instance of the MainWindow class.
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();
            _serialPort = new SerialPort("COM6", 115200, Parity.None, 8, StopBits.Two);
            _serialPort.Open();

            if (_serialPort.IsOpen == true)
            {
                this.statusBarText.Text = "Port Connected.";
                
            }
            else
                this.statusBarText.Text = "Port Not Connected.";
        }

        /// <summary>
        /// Draws indicators to show which edges are clipping skeleton data
        /// </summary>
        /// <param name="skeleton">skeleton to draw clipping information for</param>
        /// <param name="drawingContext">drawing context to draw to</param>
        private static void RenderClippedEdges(Skeleton skeleton, DrawingContext drawingContext)
        {
            if (skeleton.ClippedEdges.HasFlag(FrameEdges.Bottom))
            {
                drawingContext.DrawRectangle(
                    Brushes.Red,
                    null,
                    new Rect(0, RenderHeight - ClipBoundsThickness, RenderWidth, ClipBoundsThickness));
            }

            if (skeleton.ClippedEdges.HasFlag(FrameEdges.Top))
            {
                drawingContext.DrawRectangle(
                    Brushes.Red,
                    null,
                    new Rect(0, 0, RenderWidth, ClipBoundsThickness));
            }

            if (skeleton.ClippedEdges.HasFlag(FrameEdges.Left))
            {
                drawingContext.DrawRectangle(
                    Brushes.Red,
                    null,
                    new Rect(0, 0, ClipBoundsThickness, RenderHeight));
            }

            if (skeleton.ClippedEdges.HasFlag(FrameEdges.Right))
            {
                drawingContext.DrawRectangle(
                    Brushes.Red,
                    null,
                    new Rect(RenderWidth - ClipBoundsThickness, 0, ClipBoundsThickness, RenderHeight));
            }
        }

        /// <summary>
        /// Execute startup tasks
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void WindowLoaded(object sender, RoutedEventArgs e)
        {
            // Create the drawing group we'll use for drawing
            this.drawingGroup = new DrawingGroup();

            // Create an image source that we can use in our image control
            this.imageSource = new DrawingImage(this.drawingGroup);

            // Display the drawing using our image control
            Image.Source = this.imageSource;

            // Look through all sensors and start the first connected one.
            // This requires that a Kinect is connected at the time of app startup.
            // To make your app robust against plug/unplug, 
            // it is recommended to use KinectSensorChooser provided in Microsoft.Kinect.Toolkit
            foreach (var potentialSensor in KinectSensor.KinectSensors)
            {
                if (potentialSensor.Status == KinectStatus.Connected)
                {
                    this.sensor = potentialSensor;
                    break;
                }
            }
                        if (null != this.sensor)
            {
                // Turn on the skeleton stream to receive skeleton frames
               TransformSmoothParameters smoothingParam = new TransformSmoothParameters();
              {
                  smoothingParam.Smoothing = 0.7f;
                  smoothingParam.Correction = 0.3f;
                  smoothingParam.Prediction = 1.0f;
                  smoothingParam.JitterRadius = 1.0f;
                  smoothingParam.MaxDeviationRadius = 1.0f;
                  // Some smoothing with little latency (defaults).
                  // Only filters out small jitters.
                  // Good for gesture recognition in games.
                  //{0.5f, 0.5f, 0.5f, 0.05f, 0.04f};

                  // Smoothed with some latency.
                  // Filters out medium jitters.
                  // Good for a menu system that needs to be smooth but
                  // doesn't need the reduced latency as much as gesture recognition does.
                  //{0.5f, 0.1f, 0.5f, 0.1f, 0.1f};

                  // Very smooth, but with a lot of latency.
                  // Filters out large jitters.
                  // Good for situations where smooth data is absolutely required
                  // and latency is not an issue.
                  //{0.7f, 0.3f, 1.0f, 1.0f, 1.0f};
              };
              this.sensor.SkeletonStream.Enable(smoothingParam);

                // Add an event handler to be called whenever there is new color frame data
                this.sensor.SkeletonFrameReady += this.SensorSkeletonFrameReady;

                // Start the sensor!
                try
                {
                    this.sensor.Start();
                }
                catch (IOException)
                {
                    this.sensor = null;
                }
            }

            if (null == this.sensor)
            {
                this.statusBarText.Text = Properties.Resources.NoKinectReady;
            }



        }

        /// <summary>
        /// Execute shutdown tasks
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void WindowClosing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (null != this.sensor)
            {
                this.sensor.Stop();
                _serialPort.Close();
            }
        }

        /// <summary>
        /// Event handler for Kinect sensor's SkeletonFrameReady event
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void SensorSkeletonFrameReady(object sender, SkeletonFrameReadyEventArgs e)
        {
            Skeleton[] skeletons = new Skeleton[0];

            using (SkeletonFrame skeletonFrame = e.OpenSkeletonFrame())
            {
                if (skeletonFrame != null)
                {
                    skeletons = new Skeleton[skeletonFrame.SkeletonArrayLength];
                    skeletonFrame.CopySkeletonDataTo(skeletons);
                }
            }

            using (DrawingContext dc = this.drawingGroup.Open())
            {
                // Draw a transparent background to set the render size
                dc.DrawRectangle(Brushes.Black, null, new Rect(0.0, 0.0, RenderWidth, RenderHeight));

                if (skeletons.Length != 0)
                {                       
                    foreach (Skeleton skel in skeletons)
                    {
                        if (skel.TrackingState == SkeletonTrackingState.Tracked)
                        {
                            angA = AngleBetweenTwoVectors(skel, JointType.ShoulderRight, JointType.ElbowRight).ToString();
                            angB = AngleBetweenTwoVectors(skel, JointType.ElbowRight, JointType.WristRight).ToString();
                            angBL = AngleLeft(skel).ToString();

                            if (Time_i == 0)
                                Time_i = DateTime.Now.Ticks;
                            Time_now = DateTime.Now.Ticks;
                            if ((Time_i + 205 < Time_now) && (trackStart = true))
                            {
                                Time_i = Time_now;
                                _serialPort.Write("B" + angB + "\r");
                                _serialPort.Write("A" + angA + "\r");
                                _serialPort.Write("H" + angBL + "\r");
                                
                                statusBarText.Text = angA + "    "    +angB;
                            }            
                            this.DrawBonesAndJoints(skel, dc);
                        }
                    }
                }

                // prevent drawing outside of our render area
                this.drawingGroup.ClipGeometry = new RectangleGeometry(new Rect(0.0, 0.0, RenderWidth, RenderHeight));
            }
        }

        /// <summary>
        /// Draws a skeleton's bones and joints
        /// </summary>
        /// <param name="skeleton">skeleton to draw</param>
        /// <param name="drawingContext">drawing context to draw to</param>
        private void DrawBonesAndJoints(Skeleton skeleton, DrawingContext drawingContext)
        {
            // Render Torso
            this.DrawBone(skeleton, drawingContext, JointType.Head, JointType.ShoulderCenter);
            this.DrawBone(skeleton, drawingContext, JointType.ShoulderCenter, JointType.ShoulderLeft);
            this.DrawBone(skeleton, drawingContext, JointType.ShoulderCenter, JointType.ShoulderRight);
            this.DrawBone(skeleton, drawingContext, JointType.ShoulderCenter, JointType.Spine);
            this.DrawBone(skeleton, drawingContext, JointType.Spine, JointType.HipCenter);
  //          this.DrawBone(skeleton, drawingContext, JointType.HipCenter, JointType.HipLeft);
  //          this.DrawBone(skeleton, drawingContext, JointType.HipCenter, JointType.HipRight);

            // Left Arm
            this.DrawBone(skeleton, drawingContext, JointType.ShoulderLeft, JointType.ElbowLeft);
            this.DrawBone(skeleton, drawingContext, JointType.ElbowLeft, JointType.WristLeft);
            this.DrawBone(skeleton, drawingContext, JointType.WristLeft, JointType.HandLeft);

            // Right Arm
            this.DrawBone(skeleton, drawingContext, JointType.ShoulderRight, JointType.ElbowRight);
            this.DrawBone(skeleton, drawingContext, JointType.ElbowRight, JointType.WristRight);
            this.DrawBone(skeleton, drawingContext, JointType.WristRight, JointType.HandRight);

            // Left Leg
  //          this.DrawBone(skeleton, drawingContext, JointType.HipLeft, JointType.KneeLeft);
  //          this.DrawBone(skeleton, drawingContext, JointType.KneeLeft, JointType.AnkleLeft);
  //          this.DrawBone(skeleton, drawingContext, JointType.AnkleLeft, JointType.FootLeft);

            // Right Leg
  //          this.DrawBone(skeleton, drawingContext, JointType.HipRight, JointType.KneeRight);
  //          this.DrawBone(skeleton, drawingContext, JointType.KneeRight, JointType.AnkleRight);
  //          this.DrawBone(skeleton, drawingContext, JointType.AnkleRight, JointType.FootRight);

        /*    foreach (Joint joint in skeleton.Joints)
            {
                Brush drawBrush = null;

                if (joint.TrackingState == JointTrackingState.Tracked)
                {
                    drawBrush = this.trackedJointBrush;                    
                }
                else if (joint.TrackingState == JointTrackingState.Inferred)
                {
                    drawBrush = this.inferredJointBrush;                    
                }

                if (drawBrush != null)
                {
                    drawingContext.DrawEllipse(drawBrush, null, this.SkeletonPointToScreen(joint.Position), JointThickness, JointThickness);
                }
            }*/
        }

        /// <summary>
        /// Maps a SkeletonPoint to lie within our render space and converts to Point
        /// </summary>
        /// <param name="skelpoint">point to map</param>
        /// <returns>mapped point</returns>
        private Point SkeletonPointToScreen(SkeletonPoint skelpoint)
        {
            // Convert point to depth space.  
            // We are not using depth directly, but we do want the points in our 640x480 output resolution.
            DepthImagePoint depthPoint = this.sensor.MapSkeletonPointToDepth(
                                                                             skelpoint,
                                                                             DepthImageFormat.Resolution640x480Fps30);
            return new Point(depthPoint.X, depthPoint.Y);
        }

        /// <summary>
        /// Draws a bone line between two joints
        /// </summary>
        /// <param name="skeleton">skeleton to draw bones from</param>
        /// <param name="drawingContext">drawing context to draw to</param>
        /// <param name="jointType0">joint to start drawing from</param>
        /// <param name="jointType1">joint to end drawing at</param>
        private void DrawBone(Skeleton skeleton, DrawingContext drawingContext, JointType jointType0, JointType jointType1)
        {
            Joint joint0 = skeleton.Joints[jointType0];
            Joint joint1 = skeleton.Joints[jointType1];

            // If we can't find either of these joints, exit
            if (joint0.TrackingState == JointTrackingState.NotTracked ||
                joint1.TrackingState == JointTrackingState.NotTracked)
            {
                return;
            }

            // Don't draw if both points are inferred
            if (joint0.TrackingState == JointTrackingState.Inferred &&
                joint1.TrackingState == JointTrackingState.Inferred)
            {
                return;
            }

            // We assume all drawn bones are inferred unless BOTH joints are tracked
            Pen drawPen = this.inferredBonePen;
            if (joint0.TrackingState == JointTrackingState.Tracked && joint1.TrackingState == JointTrackingState.Tracked)
            {
                drawPen = this.trackedBonePen;
            }

            drawingContext.DrawLine(drawPen, this.SkeletonPointToScreen(joint0.Position), this.SkeletonPointToScreen(joint1.Position));
        }

        /// <summary>
        /// Handles the checking or unchecking of the seated mode combo box
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void CheckBoxSeatedModeChanged(object sender, RoutedEventArgs e)
        {
            if (null != this.sensor)
            {
                if (this.checkBoxSeatedMode.IsChecked.GetValueOrDefault())
                {
                    this.sensor.SkeletonStream.TrackingMode = SkeletonTrackingMode.Seated;
                }
                else
                {
                    this.sensor.SkeletonStream.TrackingMode = SkeletonTrackingMode.Default;
                }
            }
        }
        public double AngleBetweenTwoVectors(Skeleton skeleton, JointType jointType0,JointType jointType1)
        {
            double dotProduct = 0.0f;
            double x0=0, y0=0, z0=0, x1=0, y1=0, z1=0;
            if (jointType0 == JointType.ShoulderRight)
            {
                
                y0 = skeleton.Joints[JointType.ShoulderLeft].Position.X - skeleton.Joints[jointType0].Position.X;
                x0 = skeleton.Joints[JointType.ShoulderLeft].Position.Y - skeleton.Joints[jointType0].Position.Y;
                z0 = skeleton.Joints[JointType.ShoulderLeft].Position.Z - skeleton.Joints[jointType0].Position.Z;
            }
            else if (jointType0 == JointType.ElbowRight)
            {
                x0 = skeleton.Joints[jointType0].Position.X - skeleton.Joints[JointType.ShoulderRight].Position.X;
                y0 = skeleton.Joints[jointType0].Position.Y - skeleton.Joints[JointType.ShoulderRight].Position.Y;
                z0 = skeleton.Joints[jointType0].Position.Z - skeleton.Joints[JointType.ShoulderRight].Position.Z;
            }
            double d0 = Math.Sqrt((x0 * x0) + (y0 * y0) + (z0 * z0));
            
            if (jointType1 == JointType.ElbowRight)
            {
                x1 = skeleton.Joints[jointType1].Position.X - skeleton.Joints[JointType.ShoulderRight].Position.X;
                y1 = skeleton.Joints[jointType1].Position.Y - skeleton.Joints[JointType.ShoulderRight].Position.Y;
                z1 = skeleton.Joints[jointType1].Position.Z - skeleton.Joints[JointType.ShoulderRight].Position.Z;
            }
            else if (jointType1 == JointType.WristRight)
            {
                x1 = skeleton.Joints[jointType1].Position.X - skeleton.Joints[JointType.ElbowRight].Position.X;
                y1 = skeleton.Joints[jointType1].Position.Y - skeleton.Joints[JointType.ElbowRight].Position.Y;
                z1 = skeleton.Joints[jointType1].Position.Z - skeleton.Joints[JointType.ElbowRight].Position.Z;
            }
            double d1 = Math.Sqrt((x1 * x1) + (y1 * y1) + (z1 * z1));
            if (d0 != 0)
            {
                x0 /= d0;
                y0 /= d0;
                z0 /= d0;
            }
            if (d1 != 0)
            {
                x1 /= d1;
                y1 /= d1;
                z1 /= d1;
            }
            dotProduct = x0 * x1 + y0 * y1 + z0 * z1;//
            return (double)Math.Round((360 / (2 * Math.PI)) * Math.Acos(dotProduct)); 
        }

        public double AngleLeft(Skeleton skeleton)
        {
            double dotProduct = 0.0f;
            double x0 = 0, y0 = 0, z0 = 0, x1 = 0, y1 = 0, z1 = 0,xh = 0,yh = 0,zh = 0;
            x0 = -skeleton.Joints[JointType.ShoulderLeft].Position.X + skeleton.Joints[JointType.ElbowLeft].Position.X;
            y0 = -skeleton.Joints[JointType.ShoulderLeft].Position.Y + skeleton.Joints[JointType.ElbowLeft].Position.Y;
            z0 = -skeleton.Joints[JointType.ShoulderLeft].Position.Z + skeleton.Joints[JointType.ElbowLeft].Position.Z;
            double d0 = Math.Sqrt((x0 * x0) + (y0 * y0) + (z0 * z0));

            x1 = skeleton.Joints[JointType.WristLeft].Position.X - skeleton.Joints[JointType.ElbowLeft].Position.X;
            y1 = skeleton.Joints[JointType.WristLeft].Position.Y - skeleton.Joints[JointType.ElbowLeft].Position.Y;
            z1 = skeleton.Joints[JointType.WristLeft].Position.Z - skeleton.Joints[JointType.ElbowLeft].Position.Z;
            double d1 = Math.Sqrt((x1 * x1) + (y1 * y1) + (z1 * z1));

            xh = skeleton.Joints[JointType.WristRight].Position.X - skeleton.Joints[JointType.WristLeft].Position.X;
            zh = skeleton.Joints[JointType.WristRight].Position.Z - skeleton.Joints[JointType.WristLeft].Position.Z;
            double h0 = Math.Sqrt((xh * xh) + (zh * zh));

            if (d0 != 0)
            {
                x0 /= d0;
                y0 /= d0;
                z0 /= d0;
            }
            if (d1 != 0)
            {
                x1 /= d1;
                y1 /= d1;
                z1 /= d1;
            }
            dotProduct = x0 * x1 + y0 * y1 + z0 * z1;//
            double angle = (double)Math.Round((360 / (2 * Math.PI)) * Math.Acos(dotProduct));

            double returnValue = 0.0f;
            returnValue = h0 * 100;

            if (returnValue > 90)
                returnValue = 90;
            if (!((angle > 0) && (angle < 100)))
                returnValue = 100;
            return Math.Round(returnValue);
        }

        private void button1_Click(object sender, RoutedEventArgs e)
        {
            _serialPort.Write("S2\r");
        }

        private void button2_Click(object sender, RoutedEventArgs e)
        {
            _serialPort.Write("S1\r");
        }

   /*     private void TrackingModeChecked(object sender, RoutedEventArgs e)
        {
            if (this.TrackingMode.IsChecked.GetValueOrDefault())
            {
                _serialPort.Write("3\r");
                trackStart = true;    
            }
            else
            {
                trackStart = false;
            }
        }*/

        private void button3_Click(object sender, RoutedEventArgs e)
        {
            if (trackStart == false)
            {
                trackStart = true;
                _serialPort.Write("S3\r");
                button3.Background = Brushes.Green;
                button3.Content = "ON";
            }
            else if (trackStart == true) 
            {
                trackStart = false;
                _serialPort.Write("E\r");
                button3.Background = Brushes.LightBlue;
                button3.Content = "Kinect";
            }
        }

        private void button4_Click(object sender, RoutedEventArgs e)
        {
            _serialPort.Write("S4\r");
        }
        private void button5_Click(object sender, RoutedEventArgs e)
        {
            _serialPort.Write("S5\r");
        }

    }
}