/*******************************************************

This code is designed to control two motors connected to 
the 9s12-based Esduino board.

Sources: Code by Valvano 
(http://www.ece.utexas.edu/~valvano/Starterfiles/)

    -> SCI.c example code
    -> PLL.c example code
    -> ADC.c example code
    
    Jung Soo Kim, Nika Zolfaghari, James Andrew Smith; May 8th, 2013


********************************************************/

#include <hidef.h>      
#include "derivative.h"     
#include <mc9s12c32.h>
#include "math.h"                          
#include "SCI.h"
#include "pll.h"
#include "ADC.h"

#pragma LINK_INFO DERIVATIVE "mc9s12c32"

#define ADC_TIME_CONSTANT   1               //1024 x ADC_TIME_CONSTANT for acquiring input ADC measurement
#define SCI_TIME_CONSTANT   100             //1024 x SCI_TIME_CONSTANT for sending packets via SCI
#define TIME_CONSTANT       0.001024    
#define KD_CONSTANT         0.001024
#define DIVISOR             10              //To allow sending floats via SCI
#define START_A             300             // ADC value of Potentiometer at its rest position
#define START_B             368             // ADC value of Potentiometer at its rest position


void PWM_Init(void); 
void DIR_Init(void);
void RTI_Init(void);
void testPin();
void OutCRLF(void);
void motionArm();
void getParam(void);
void shutOff(void);

const float  PI = 3.14159265358979323846f;
float angA = -1, angB = -1;                 //current measurement of angle, voltage, current
float angA_i = 0, angB_i = 0;                                          //previous angle
float errorA, errorB, errorA_i = 0, errorB_i = 0;                      //current and previous error
float errorAT, errorBT;                                                //cumulative error
float DTY1 = 0, DTY5 = 0;                                              //duty cycle
float time = 0;

float Kp_A, Kd_A, Kp_B, Kd_B, Ki_A, Ki_B;

short startBit =0;
short startMode3 = 1;                                                     //startbit of module
unsigned long CountRTI = 0;
unsigned int CountIR = 0;                                             //Counter of RTI
unsigned int CountClock = 0;
float desAngA = 0, desAngB = 0;                                        //destination angle requested
int RunTime = 0;                                                       //total runtime of module
float dTime = 0;                                                       //runtime of each move when 2 destinations are specified
int motorA, motorB, posA1, posA2, posB1, posB2, cyc, motorAi, motorBi;
int timeCount;
float sbit;                                                             //switch bit to indicate change between 2 destinations
float cycFactor = 0;                                                    //cycloidal factor for angle
float cycFactorD = 0;    

short mode3_A=10, mode3_Ai = 10;
short mode3_B=10, mode3_Bi = 10;
short mode3_H = 0;
int testTemp = 0;

void main(void) {

  PLL_Init();                           //Initiallize PLL
  ADC_Init();                           //Initiallize ADC
  RTI_Init();                           //Initiallize RTI


  SCI_Init(115200);                     //Initiallize SCI BaudRate of 115200
  
  PWM_Init();                           //Initiallize PWM
  DIR_Init();                           //Initiallize directions for port M

  PWMDTY1= 0;                           //PWM duty cycle A
  PWMDTY5= 0;                           //PWM duty cycle B

  sbit = 1;

  errorA = 0;
  errorB = 0;
  errorA_i = 0;
  errorB_i = 0;
  errorAT =0;
  errorBT = 0;
  desAngA = 0;
  desAngB = 0;
  DTY1 = 0;
  DTY5 = 0;

  motorAi = START_A;
  motorBi = START_B; 
  motorA = START_A;
  motorB = START_B;

  timeCount = 0;
 
  asm cli; 
 
  for(;;) {    
  
  if(startBit==0){
    getParam();
  } 
  else if(startBit ==3){
      Kp_A = 10;
      Kd_A = 1;
      Ki_A = 0;
      Kp_B = 1;
      Kd_B = 0;
      Ki_B = 0;
      
    } 
    else{
      startBit=0;
    } 
  }
    
}


/*************************************************************************
   shutOff():finish up the reqeusted test and send end token via SCI
*************************************************************************/

void shutOff(){
    startBit = 0;
    PWMDTY1 = 0;
    PWMDTY5 = 0;
    CountRTI = 0;
    errorAT = 0;
    errorBT = 0;
    startBit = 0;
    mode3_A=0;
    mode3_B=0;
    mode3_H = 0;
    sbit  = 1;
}
/*************************************************************************
   getParam(): receive all the variables sent via SCI from MATLAB
                  the number of parameters and their use needs to be matched
                  with sending of MATLAB
**************************************************************************/

void getParam(){
  
  startBit = SCI_InMotor('S');
  if (startBit == 4490)
   shutOff();

}

/*************************************************************************
***   motionArm():Change motor's speed by changing its PWM and direction bit
                  The calculation is based on PID controller
                  and cycoidal motion                                     
 **************************************************************************/
 void motionArm(){
  
  desAngA = (motorA-motorAi)*cycFactor + motorAi;              //destination angle of motor A
  desAngB = (motorB-motorBi)*cycFactor + motorBi;              //destination angle of motor B
  
  errorA = ((desAngA)-angA);                                    //error of angle A
  errorAT = (errorAT + errorA);                                //cumulative error A
                           
  if (errorA_i== 0)   DTY1 = errorA* Kp_A;
  else    DTY1 = errorA* Kp_A + (errorA-errorA_i)*KD_CONSTANT*(Kd_A/(TIME_CONSTANT))*cycFactorD + Ki_A*errorAT*TIME_CONSTANT;       
  
  //direction change for motor A        
  if(DTY1 >0)  {
    PTM_PTM3 = 0;
  } else {   
      PTM_PTM3 = 1;
      if(startBit==3)
      DTY1=DTY1*-0.5;
      else
      DTY1 = DTY1*-1;
  }        
  
  //In case the duty cycle saturates
  if (DTY1>255) DTY1 = 255;
  PWMDTY1 = DTY1; 


  errorB = ((desAngB)-angB);                                  //error of angle B
  errorBT = (errorBT + errorB);                               //cumulative error B
  
  //calculate PID for motor B     
  if (errorB_i== 0)     DTY5 = errorB* Kp_B;
  else     DTY5 = errorB* Kp_B + (errorB-errorB_i)*KD_CONSTANT*(Kd_B/(TIME_CONSTANT))*cycFactorD + Ki_B*errorBT*TIME_CONSTANT;   

  //direction change for motor B  
  if(DTY5 >0)  {
    PTM_PTM5 = 0;
  } else {
    PTM_PTM5 = 1;
    if(startBit==3)
      DTY5=DTY5*-0.5;
      else
      
    DTY5 = DTY5*-1;
  }       

  //In case the duty cycle saturates
  if (DTY5>255) DTY5 = 255;  
  PWMDTY5 =DTY5;
         
  angA_i = angA;                                              //remember the previous angle
  angB_i = angB;  
  errorA_i = errorA;                                          //remember the previous error
  errorB_i = errorB; 
}

/*************************************************************************
***   DIR_Init():Iniatiate the direction of port M
**************************************************************************/
void DIR_Init(void){
  
  SPICR1 = 0x00; //turn off SPI
  DDRM = 0x2F;   //PM5 and PM3 as outputs, rest inputs  
}
/*************************************************************************
***   PWM_Init():Initiate PWM settings
                 PTT5, PTT4, PTT1 and PTT0 is used to output PWM
                 using clock SA of46.875kHz
                 Period is 255.
**************************************************************************/
void PWM_Init(void){
  MODRR = 0x26;    // PTT5,PTT4,PTT1,PTT0 associated with PWM
  PWME = 0x26;	   // enable channels 0,1,4,5
  PWMPOL = 0x26;   // PTT5,PTT4,PTT1,PPT0 high then low 
  PWMCLK=0x26;
  PWMPRCLK = 0x76; 
  PWMCAE = 0;      // left aligned mode 
  PWMCTL = 0x00;
  PWMSCLA =4; 
  PWMSCLB = 3;    
  
  PWMPER1 = 255;   // period1
  PWMDTY1 = 0;     // duty cycle1, off
  PWMPER2 = 255;
  PWMDTY2 = 0;
  PWMPER5 = 255;   // period1
  PWMDTY5 = 0;     // duty cycle1, off
  PTM_PTM2 = 0;
  PTM_PTM3 = 0;    // direction bit initialized
  PTM_PTM5 = 0;    // direction bit initialized

}

/*************************************************************************
***   RTI_Init(): Initiate the Real Time Interupt (RTI)
**************************************************************************/
void RTI_Init(void){
  asm sei        // Make ritual atomic 

  CRGINT = 0x80; // RTIE=1 enable rti
  RTICTL = 0x31; // period=125ns*4096*1=512us or 1953.125Hz

  asm cli
}

void testPin(){
  
  if(PTT_PTT6 == 1) {
    
    PTT_PTT6 = 0;;
   }
    else {
    
    PTT_PTT6 = 1;
  }
}  
 
       
 void interrupt 7 handler(){
 

  CRGFLG = 0x80;                                    // acknowledge, clear RTIF flag


  angA = ADC_In(0x80);                              // acquire ADC measurement, (angle, current, voltage)
  angB = ADC_In(0x81);
  
  if((startBit ==3)){
      
        if(CountRTI%20==0){
   
        testTemp = SCI_InMotor('B');
        
        if (testTemp == 1112)
            shutOff();
        else{
          
            mode3_Bi = mode3_B;
            mode3_Ai = mode3_A;
            
            if((testTemp!=4490) && (testTemp<180) && (testTemp>0))
              mode3_B =testTemp; 
            
            testTemp = SCI_InMotor('A');
            if((testTemp!=4490) && (testTemp<160) && (testTemp>0))
              mode3_A =testTemp;       
            
            testTemp = SCI_InMotor('H');
            if((testTemp!=4490) && (testTemp<180) && (testTemp>0))
              mode3_H =testTemp;
            }
            
          if(((mode3_A<mode3_Ai-30)||(mode3_A>mode3_Ai+60)))         mode3_A=mode3_Ai;
          if(((mode3_B<mode3_Bi-30)||(mode3_B>mode3_Bi+60)))         mode3_B=mode3_Bi;
        }
     
        motorAi=motorA;
        motorBi=motorB;

        motorA = ((1024/225)*mode3_A)+START_A;
        motorB = ((1024/225)*mode3_B)+START_B;

        cycFactor=1;
        cycFactorD=1;
        
        motionArm();
      
        if(CountRTI>20*60000) {
          CountRTI=0;
        }
        
        CountRTI++;
  }                     
 
 }


void OutCRLF(void){                         //add CR and LF on the end of SCI transmission
  SCI_OutChar(CR);
  SCI_OutChar(LF);
}       

